@@ -71,14 +71,14 @@ It is used for TCP/IP devices."
   "Regexp for date time format in ls output."))
 
 (defconst tramp-adb-ls-date-regexp
-  (tramp-compat-rx
+  (rx
    blank (regexp tramp-adb-ls-date-year-regexp)
    blank (regexp tramp-adb-ls-date-time-regexp)
    blank)
   "Regexp for date format in ls output.")
 
 (defconst tramp-adb-ls-toolbox-regexp
-  (tramp-compat-rx
+  (rx
    bol (* blank) (group (+ (any ".-" alpha)))			; \1 permissions
    (? (+ blank) (+ digit))			      ; links (Android 7/toybox)
    (* blank) (group (+ (not blank)))				; \2 username
@@ -130,6 +130,7 @@ It is used for TCP/IP devices."
     (file-equal-p . tramp-handle-file-equal-p)
     (file-executable-p . tramp-adb-handle-file-executable-p)
     (file-exists-p . tramp-adb-handle-file-exists-p)
+    (file-group-gid . tramp-handle-file-group-gid)
     (file-in-directory-p . tramp-handle-file-in-directory-p)
     (file-local-copy . tramp-adb-handle-file-local-copy)
     (file-locked-p . tramp-handle-file-locked-p)
@@ -153,6 +154,7 @@ It is used for TCP/IP devices."
     (file-symlink-p . tramp-handle-file-symlink-p)
     (file-system-info . tramp-adb-handle-file-system-info)
     (file-truename . tramp-handle-file-truename)
+    (file-user-uid . tramp-handle-file-user-uid)
     (file-writable-p . tramp-adb-handle-file-writable-p)
     (find-backup-file-name . tramp-handle-find-backup-file-name)
     ;; `get-file-buffer' performed by default handler.
@@ -207,8 +209,10 @@ It is used for TCP/IP devices."
 First arg specifies the OPERATION, second arg is a list of
 arguments to pass to the OPERATION."
   (if-let ((fn (assoc operation tramp-adb-file-name-handler-alist)))
-      (save-match-data (apply (cdr fn) args))
-    (tramp-run-real-handler operation args)))
+      (prog1 (save-match-data (apply (cdr fn) args))
+	(setq tramp-debug-message-fnh-function (cdr fn)))
+    (prog1 (tramp-run-real-handler operation args)
+      (setq tramp-debug-message-fnh-function operation))))
 
 ;;;###tramp-autoload
 (tramp--with-startup
@@ -271,7 +275,7 @@ arguments to pass to the OPERATION."
   (with-current-buffer (tramp-get-buffer vec)
     (goto-char (point-min))
     (let (file-properties)
-      (while (re-search-forward tramp-adb-ls-toolbox-regexp nil t)
+      (while (search-forward-regexp tramp-adb-ls-toolbox-regexp nil t)
 	(let* ((mod-string (match-string 1))
 	       (is-dir (eq ?d (aref mod-string 0)))
 	       (is-symlink (eq ?l (aref mod-string 0)))
@@ -317,7 +321,7 @@ arguments to pass to the OPERATION."
 		       (tramp-shell-quote-argument localname)))
 	;; We insert also filename/. and filename/.., because "ls"
 	;; doesn't on some file systems, like "sdcard".
-	(unless (re-search-backward (rx "." eol) nil t)
+	(unless (search-backward-regexp (rx "." eol) nil t)
 	  (narrow-to-region (point-max) (point-max))
 	  (tramp-adb-send-command
 	   v (format "%s -d -a -l %s %s | cat"
@@ -327,8 +331,7 @@ arguments to pass to the OPERATION."
 		     (tramp-shell-quote-argument
 		      (tramp-compat-file-name-concat localname ".."))))
 	  (tramp-compat-replace-regexp-in-region
-	   (tramp-compat-rx (literal (tramp-compat-file-name-unquote
-				      (file-name-as-directory localname))))
+	   (rx (literal (file-name-unquote (file-name-as-directory localname))))
 	   "" (point-min))
 	  (widen)))
       (tramp-adb-sh-fix-ls-output)
@@ -366,14 +369,12 @@ Emacs dired can't find files."
     (goto-char (point-min))
     (while
 	(search-forward-regexp
-	 (tramp-compat-rx
-	  blank (group blank (regexp tramp-adb-ls-date-year-regexp) blank))
+	 (rx blank (group blank (regexp tramp-adb-ls-date-year-regexp) blank))
 	 nil t)
       (replace-match "0\\1" "\\1" nil)
       ;; Insert missing "/".
       (when (looking-at-p
-	     (tramp-compat-rx
-	      (regexp tramp-adb-ls-date-time-regexp) (+ blank) eol))
+	     (rx (regexp tramp-adb-ls-date-time-regexp) (+ blank) eol))
 	(end-of-line)
 	(insert "/")))
     ;; Sort entries.
@@ -393,12 +394,10 @@ Emacs dired can't find files."
 (defun tramp-adb-ls-output-time-less-p (a b)
   "Sort \"ls\" output by time, descending."
   (let (time-a time-b)
-    ;; Once we can assume Emacs 27 or later, the two calls
-    ;; (apply #'encode-time X) can be replaced by (encode-time X).
     (string-match tramp-adb-ls-date-regexp a)
-    (setq time-a (apply #'encode-time (parse-time-string (match-string 0 a))))
+    (setq time-a (encode-time (parse-time-string (match-string 0 a))))
     (string-match tramp-adb-ls-date-regexp b)
-    (setq time-b (apply #'encode-time (parse-time-string (match-string 0 b))))
+    (setq time-b (encode-time (parse-time-string (match-string 0 b))))
     (time-less-p time-b time-a)))
 
 (defun tramp-adb-ls-output-name-less-p (a b)
@@ -411,20 +410,11 @@ Emacs dired can't find files."
 
 (defun tramp-adb-handle-make-directory (dir &optional parents)
   "Like `make-directory' for Tramp files."
-  (setq dir (expand-file-name dir))
-  (with-parsed-tramp-file-name dir nil
-    (when (and (null parents) (file-exists-p dir))
-      (tramp-error v 'file-already-exists dir))
-    (when parents
-      (let ((par (expand-file-name ".." dir)))
-	(unless (file-directory-p par)
-	  (make-directory par parents))))
-    (tramp-flush-directory-properties v localname)
-    (unless (or (tramp-adb-send-command-and-check
-		 v (format "mkdir -m %#o %s"
-			   (default-file-modes)
-			   (tramp-shell-quote-argument localname)))
-		(and parents (file-directory-p dir)))
+  (tramp-skeleton-make-directory dir parents
+    (unless (tramp-adb-send-command-and-check
+	     v (format "mkdir -m %#o %s"
+		       (default-file-modes)
+		       (tramp-shell-quote-argument localname)))
       (tramp-error v 'file-error "Couldn't make directory %s" dir))))
 
 (defun tramp-adb-handle-delete-directory (directory &optional recursive trash)
@@ -438,14 +428,10 @@ Emacs dired can't find files."
 
 (defun tramp-adb-handle-delete-file (filename &optional trash)
   "Like `delete-file' for Tramp files."
-  (setq filename (expand-file-name filename))
-  (with-parsed-tramp-file-name filename nil
-    (tramp-flush-file-properties v localname)
-    (if (and delete-by-moving-to-trash trash)
-	(move-file-to-trash filename)
-      (tramp-adb-barf-unless-okay
-       v (format "rm %s" (tramp-shell-quote-argument localname))
-       "Couldn't delete %s" filename))))
+  (tramp-skeleton-delete-file filename trash
+    (tramp-adb-barf-unless-okay
+     v (format "rm %s" (tramp-shell-quote-argument localname))
+     "Couldn't delete %s" filename)))
 
 (defun tramp-adb-handle-file-name-all-completions (filename directory)
   "Like `file-name-all-completions' for Tramp files."
@@ -477,7 +463,7 @@ Emacs dired can't find files."
       ;; "adb pull ..." does not always return an error code.
       (unless
 	  (and (tramp-adb-execute-adb-command
-		v "pull" (tramp-compat-file-name-unquote localname) tmpfile)
+		v "pull" (file-name-unquote localname) tmpfile)
 	       (file-exists-p tmpfile))
 	(ignore-errors (delete-file tmpfile))
 	(tramp-error
@@ -518,9 +504,9 @@ Emacs dired can't find files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (with-tramp-file-property v localname "file-writable-p"
       (if (file-exists-p filename)
+	  ;; Examine `file-attributes' cache to see if request can be
+	  ;; satisfied without remote operation.
 	  (if (tramp-file-property-p v localname "file-attributes")
-	      ;; Examine `file-attributes' cache to see if request can
-	      ;; be satisfied without remote operation.
 	      (tramp-check-cached-permissions v ?w)
 	    (tramp-adb-send-command-and-check
 	     v (format "test -w %s" (tramp-shell-quote-argument localname))))
@@ -552,8 +538,7 @@ Emacs dired can't find files."
 		 "Moving tmp file `%s' to `%s'" tmpfile filename)
 	  (unwind-protect
 	      (unless (tramp-adb-execute-adb-command
-		       v "push" tmpfile
-		       (tramp-compat-file-name-unquote localname))
+		       v "push" tmpfile (file-name-unquote localname))
 		(tramp-error v 'file-error "Cannot write: `%s'" filename))
 	    (delete-file tmpfile)))))))
 
@@ -568,11 +553,7 @@ Emacs dired can't find files."
 (defun tramp-adb-handle-set-file-times (filename &optional time flag)
   "Like `set-file-times' for Tramp files."
   (tramp-skeleton-set-file-modes-times-uid-gid filename
-    (let ((time (if (or (null time)
-			(tramp-compat-time-equal-p time tramp-time-doesnt-exist)
-			(tramp-compat-time-equal-p time tramp-time-dont-know))
-		    (current-time)
-		  time))
+    (let ((time (tramp-defined-time time))
 	  (nofollow (if (eq flag 'nofollow) "-h" ""))
 	  (quoted-name (tramp-shell-quote-argument localname)))
       ;; Older versions of toybox 'touch' mishandle nanoseconds and/or
@@ -658,8 +639,8 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		  (tramp-flush-file-properties v localname)
 		  (unless (tramp-adb-execute-adb-command
 			   v "push"
-			   (tramp-compat-file-name-unquote filename)
-			   (tramp-compat-file-name-unquote localname))
+			   (file-name-unquote filename)
+			   (file-name-unquote localname))
 		    (tramp-error
 		     v 'file-error
 		     "Cannot copy `%s' `%s'" filename newname)))))))))
@@ -725,11 +706,6 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   "Strings to return by `process-file' in case of signals."
   (with-tramp-connection-property vec "signal-strings"
     (let ((default-directory (tramp-make-tramp-file-name vec 'noloc))
-	  ;; `shell-file-name' and `shell-command-switch' are needed
-	  ;; for Emacs < 27.1, which doesn't support connection-local
-	  ;; variables in `shell-command'.
-	  (shell-file-name "/system/bin/sh")
-	  (shell-command-switch "-c")
 	  process-file-return-signal-string signals result)
       (dotimes (i 128) (push (format "Signal %d" i) result))
       (setq result (reverse result)
@@ -762,7 +738,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       ;; Determine input.
       (if (null infile)
 	  (setq input (tramp-get-remote-null-device v))
-	(setq infile (tramp-compat-file-name-unquote (expand-file-name infile)))
+	(setq infile (file-name-unquote (expand-file-name infile)))
 	(if (tramp-equal-remote default-directory infile)
 	    ;; INFILE is on the same remote host.
 	    (setq input (tramp-unquote-file-local-name infile))
@@ -938,9 +914,9 @@ implementation will be used."
 		 (i 0)
 		 p)
 
-	    (when (string-match-p (tramp-compat-rx multibyte) command)
+	    (when (string-match-p (rx multibyte) command)
 	      (tramp-error
-	       v 'file-error "Cannot apply multi-byte command `%s'" command))
+	       v 'file-error "Cannot apply multibyte command `%s'" command))
 
 	    (while (get-process name1)
 	      ;; NAME must be unique as process name.
@@ -1022,6 +998,7 @@ implementation will be used."
 			    ;; deleted.
 			    (when (bufferp stderr)
 			      (ignore-errors
+				(tramp-taint-remote-process-buffer stderr)
 				(with-current-buffer stderr
 			          (insert-file-contents-literally
 			           remote-tmpstderr 'visit)))
@@ -1133,7 +1110,7 @@ error and non-nil on success."
 
 (defun tramp-adb-send-command (vec command &optional neveropen nooutput)
   "Send the COMMAND to connection VEC."
-  (if (string-match-p (tramp-compat-rx multibyte) command)
+  (if (string-match-p (rx multibyte) command)
       ;; Multibyte codepoints with four bytes are not supported at
       ;; least by toybox.
 
@@ -1157,12 +1134,12 @@ error and non-nil on success."
 	  ;; We can't use stty to disable echo of command.  stty is said
 	  ;; to be added to toybox 0.7.6.  busybox shall have it, but this
 	  ;; isn't used any longer for Android.
-	  (delete-matching-lines (tramp-compat-rx bol (literal command) eol))
+	  (delete-matching-lines (rx bol (literal command) eol))
 	  ;; When the local machine is W32, there are still trailing ^M.
 	  ;; There must be a better solution by setting the correct coding
 	  ;; system, but this requires changes in core Tramp.
 	  (goto-char (point-min))
-	  (while (re-search-forward (rx (+ "\r") eol) nil t)
+	  (while (search-forward-regexp (rx (+ "\r") eol) nil t)
 	    (replace-match "" nil nil)))))))
 
 (defun tramp-adb-send-command-and-check (vec command &optional exit-status)
@@ -1206,12 +1183,12 @@ FMT and ARGS are passed to `error'."
 	  (let ((inhibit-read-only t))
 	    (goto-char (point-min))
 	    ;; ADB terminal sends "^H" sequences.
-	    (when (re-search-forward (rx "<" (+ "\b")) (line-end-position) t)
+	    (when (search-forward-regexp (rx "<" (+ "\b")) (line-end-position) t)
 	      (forward-line 1)
 	      (delete-region (point-min) (point)))
 	    ;; Delete the prompt.
             (goto-char (point-min))
-            (when (re-search-forward prompt (line-end-position) t)
+            (when (search-forward-regexp prompt (line-end-position) t)
               (forward-line 1)
               (delete-region (point-min) (point)))
 	    (when (tramp-search-regexp prompt)
@@ -1231,108 +1208,109 @@ connection if a previous connection has died for some reason."
   (unless (tramp-connectable-p vec)
     (throw 'non-essential 'non-essential))
 
-  (let* ((buf (tramp-get-connection-buffer vec))
-	 (p (get-buffer-process buf))
-	 (host (tramp-file-name-host vec))
-	 (user (tramp-file-name-user vec))
-         (device (tramp-adb-get-device vec)))
+  (with-tramp-debug-message vec "Opening connection"
+    (let* ((buf (tramp-get-connection-buffer vec))
+	   (p (get-buffer-process buf))
+	   (host (tramp-file-name-host vec))
+	   (user (tramp-file-name-user vec))
+           (device (tramp-adb-get-device vec)))
 
-    ;; Maybe we know already that "su" is not supported.  We cannot
-    ;; use a connection property, because we have not checked yet
-    ;; whether it is still the same device.
-    (when (and user (not (tramp-get-file-property vec "/" "su-command-p" t)))
-      (tramp-error vec 'file-error "Cannot switch to user `%s'" user))
+      ;; Maybe we know already that "su" is not supported.  We cannot
+      ;; use a connection property, because we have not checked yet
+      ;; whether it is still the same device.
+      (when (and user (not (tramp-get-file-property vec "/" "su-command-p" t)))
+	(tramp-error vec 'file-error "Cannot switch to user `%s'" user))
 
-    (unless (process-live-p p)
-      (save-match-data
-	(when (and p (processp p)) (delete-process p))
-	(if (tramp-string-empty-or-nil-p device)
-	    (tramp-error vec 'file-error "Device %s not connected" host))
-	(with-tramp-progress-reporter vec 3 "Opening adb shell connection"
-	  (let* ((coding-system-for-read 'utf-8-dos) ; Is this correct?
-		 (process-connection-type tramp-process-connection-type)
-		 (args (tramp-expand-args
-			vec 'tramp-login-args ?d (or device "")))
-		 (p (let ((default-directory
-			    tramp-compat-temporary-file-directory))
-		      (apply #'start-process (tramp-get-connection-name vec) buf
-			     tramp-adb-program args)))
-		 (prompt (md5 (concat (prin1-to-string process-environment)
-				      (current-time-string)))))
-	    (tramp-message
-	     vec 6 "%s" (string-join (process-command p) " "))
-	    ;; Wait for initial prompt.  On some devices, it needs an
-	    ;; initial RET, in order to get it.
-            (sleep-for 0.1)
-	    (tramp-send-string vec tramp-rsh-end-of-line)
-	    (tramp-adb-wait-for-output p 30)
-	    (unless (process-live-p p)
-	      (tramp-error vec 'file-error "Terminated!"))
+      (unless (process-live-p p)
+	(save-match-data
+	  (when (and p (processp p)) (delete-process p))
+	  (if (tramp-string-empty-or-nil-p device)
+	      (tramp-error vec 'file-error "Device %s not connected" host))
+	  (with-tramp-progress-reporter vec 3 "Opening adb shell connection"
+	    (let* ((coding-system-for-read 'utf-8-dos) ; Is this correct?
+		   (process-connection-type tramp-process-connection-type)
+		   (args (tramp-expand-args
+			  vec 'tramp-login-args ?d (or device "")))
+		   (p (let ((default-directory
+			     tramp-compat-temporary-file-directory))
+			(apply
+			 #'start-process (tramp-get-connection-name vec) buf
+			 tramp-adb-program args)))
+		   (prompt (md5 (concat (prin1-to-string process-environment)
+					(current-time-string)))))
+	      ;; Wait for initial prompt.  On some devices, it needs
+	      ;; an initial RET, in order to get it.
+              (sleep-for 0.1)
+	      (tramp-send-string vec tramp-rsh-end-of-line)
+	      (tramp-adb-wait-for-output p 30)
+	      (unless (process-live-p p)
+		(tramp-error vec 'file-error "Terminated!"))
 
-	    ;; Set sentinel and query flag.  Initialize variables.
-	    (set-process-sentinel p #'tramp-process-sentinel)
-	    (process-put p 'tramp-vector vec)
-	    (process-put p 'adjust-window-size-function #'ignore)
-	    (set-process-query-on-exit-flag p nil)
+	      ;; Set sentinel.  Initialize variables.
+	      (set-process-sentinel p #'tramp-process-sentinel)
+	      (tramp-post-process-creation p vec)
 
-	    ;; Set connection-local variables.
-	    (tramp-set-connection-local-variables vec)
+	      ;; Set connection-local variables.
+	      (tramp-set-connection-local-variables vec)
 
-	    ;; Change prompt.
-	    (tramp-set-connection-property
-	     p "prompt" (tramp-compat-rx "///" (literal prompt) "#$"))
-	    (tramp-adb-send-command
-	     vec (format "PS1=\"///\"\"%s\"\"#$\"" prompt))
+	      ;; Change prompt.
+	      (tramp-set-connection-property
+	       p "prompt" (rx "///" (literal prompt) "#$"))
+	      (tramp-adb-send-command
+	       vec (format "PS1=\"///\"\"%s\"\"#$\"" prompt))
 
-	    ;; Disable line editing.
-	    (tramp-adb-send-command
-	     vec "set +o vi +o vi-esccomplete +o vi-tabcomplete +o emacs")
+	      ;; Disable line editing.
+	      (tramp-adb-send-command
+	       vec "set +o vi +o vi-esccomplete +o vi-tabcomplete +o emacs")
 
-	    ;; Dump option settings in the traces.
-	    (when (>= tramp-verbose 9)
-	      (tramp-adb-send-command vec "set -o"))
+	      ;; Dump option settings in the traces.
+	      (when (>= tramp-verbose 9)
+		(tramp-adb-send-command vec "set -o"))
 
-	    ;; Check whether the properties have been changed.  If
-	    ;; yes, this is a strong indication that we must expire all
-	    ;; connection properties.  We start again.
-	    (tramp-message vec 5 "Checking system information")
-	    (tramp-adb-send-command
-	     vec
-	     (concat
-	      "echo \\\"`getprop ro.product.model` "
-	      "`getprop ro.product.version` "
-	      "`getprop ro.build.version.release`\\\""))
-	    (let ((old-getprop (tramp-get-connection-property vec "getprop"))
-		  (new-getprop
-		   (tramp-set-connection-property
-		    vec "getprop"
-		    (with-current-buffer (tramp-get-connection-buffer vec)
-		      ;; Read the expression.
-		      (goto-char (point-min))
-		      (read (current-buffer))))))
-	      (when (and (stringp old-getprop)
-			 (not (string-equal old-getprop new-getprop)))
-		(tramp-message
-		 vec 3
-		 "Connection reset, because remote host changed from `%s' to `%s'"
-		 old-getprop new-getprop)
-		(tramp-cleanup-connection vec t)
-		(tramp-adb-maybe-open-connection vec)))
+	      ;; Check whether the properties have been changed.  If
+	      ;; yes, this is a strong indication that we must expire
+	      ;; all connection properties.  We start again.
+	      (tramp-message vec 5 "Checking system information")
+	      (tramp-adb-send-command
+	       vec
+	       (concat
+		"echo \\\"`getprop ro.product.model` "
+		"`getprop ro.product.version` "
+		"`getprop ro.build.version.release`\\\""))
+	      (let ((old-getprop (tramp-get-connection-property vec "getprop"))
+		    (new-getprop
+		     (tramp-set-connection-property
+		      vec "getprop"
+		      (with-current-buffer (tramp-get-connection-buffer vec)
+			;; Read the expression.
+			(goto-char (point-min))
+			(read (current-buffer))))))
+		(when (and (stringp old-getprop)
+			   (not (string-equal old-getprop new-getprop)))
+		  (tramp-message
+		   vec 3
+		   (concat
+		    "Connection reset, because remote host changed "
+		    "from `%s' to `%s'")
+		   old-getprop new-getprop)
+		  (tramp-cleanup-connection vec t)
+		  (tramp-adb-maybe-open-connection vec)))
 
-	    ;; Change user if indicated.
-	    (when user
-	      (tramp-adb-send-command vec (format "su %s" user))
-	      (unless (tramp-adb-send-command-and-check vec nil)
-		(delete-process p)
-		;; Do not flush, we need the nil value.
-		(tramp-set-file-property vec "/" "su-command-p" nil)
-		(tramp-error
-		 vec 'file-error "Cannot switch to user `%s'" user)))
+	      ;; Change user if indicated.
+	      (when user
+		(tramp-adb-send-command vec (format "su %s" user))
+		(unless (tramp-adb-send-command-and-check vec nil)
+		  (delete-process p)
+		  ;; Do not flush, we need the nil value.
+		  (tramp-set-file-property vec "/" "su-command-p" nil)
+		  (tramp-error
+		   vec 'file-error "Cannot switch to user `%s'" user)))
 
-	    ;; Mark it as connected.
-	    (tramp-set-connection-property p "connected" t)))))))
+	      ;; Mark it as connected.
+	      (tramp-set-connection-property p "connected" t))))))))
 
 ;;; Default connection-local variables for Tramp.
+
 (defconst tramp-adb-connection-local-default-shell-variables
   '((shell-file-name . "/system/bin/sh")
     (shell-command-switch . "-c"))
