@@ -146,7 +146,7 @@ They are completed by \"M-x TAB\" only when encryption support is enabled."
 If NAME doesn't belong to an encrypted remote directory, return nil."
   (catch 'crypt-file-name-p
     (and tramp-crypt-enabled (stringp name)
-	 (not (tramp-compat-file-name-quoted-p name))
+	 (not (file-name-quoted-p name))
 	 (not (string-suffix-p tramp-crypt-encfs-config name))
 	 ;; No lock file name.
 	 (not (string-prefix-p ".#" (file-name-nondirectory name)))
@@ -183,6 +183,7 @@ If NAME doesn't belong to an encrypted remote directory, return nil."
     (file-equal-p . tramp-handle-file-equal-p)
     (file-executable-p . tramp-crypt-handle-file-executable-p)
     (file-exists-p . tramp-crypt-handle-file-exists-p)
+    ;; `file-group-gid' performed by default-handler.
     (file-in-directory-p . tramp-handle-file-in-directory-p)
     (file-local-copy . tramp-handle-file-local-copy)
     (file-locked-p . tramp-crypt-handle-file-locked-p)
@@ -206,6 +207,7 @@ If NAME doesn't belong to an encrypted remote directory, return nil."
     (file-symlink-p . tramp-handle-file-symlink-p)
     (file-system-info . tramp-crypt-handle-file-system-info)
     ;; `file-truename' performed by default handler.
+    ;; `file-user-uid' performed by default-handler.
     (file-writable-p . tramp-crypt-handle-file-writable-p)
     (find-backup-file-name . tramp-handle-find-backup-file-name)
     ;; `get-file-buffer' performed by default handler.
@@ -279,8 +281,10 @@ arguments to pass to the OPERATION."
 	    (apply #'tramp-crypt-file-name-for-operation operation args))
 	   (fn (and (tramp-crypt-file-name-p filename)
 		    (assoc operation tramp-crypt-file-name-handler-alist))))
-      (save-match-data (apply (cdr fn) args))
-    (tramp-crypt-run-real-handler operation args)))
+      (prog1 (save-match-data (apply (cdr fn) args))
+	(setq tramp-debug-message-fnh-function (cdr fn)))
+    (prog1 (tramp-crypt-run-real-handler operation args)
+      (setq tramp-debug-message-fnh-function operation))))
 
 ;;;###tramp-autoload
 (progn (defun tramp-register-crypt-file-name-handler ()
@@ -312,74 +316,75 @@ connection if a previous connection has died for some reason."
   ;; For password handling, we need a process bound to the connection
   ;; buffer.  Therefore, we create a dummy process.  Maybe there is a
   ;; better solution?
-  (unless (get-buffer-process (tramp-get-connection-buffer vec))
-    (let ((p (make-network-process
-	      :name (tramp-get-connection-name vec)
-	      :buffer (tramp-get-connection-buffer vec)
-	      :server t :host 'local :service t :noquery t)))
-      (process-put p 'tramp-vector vec)
-      (set-process-query-on-exit-flag p nil)))
+  (with-tramp-debug-message vec "Opening connection"
+    (unless (get-buffer-process (tramp-get-connection-buffer vec))
+      (let ((p (make-network-process
+		:name (tramp-get-connection-name vec)
+		:buffer (tramp-get-connection-buffer vec)
+		:server t :host 'local :service t :noquery t)))
+	(tramp-post-process-creation p vec)))
 
-  ;; The following operations must be performed without
-  ;; `tramp-crypt-file-name-handler'.
-  (let* (tramp-crypt-enabled
-	 ;; Don't check for a proper method.
-	 (non-essential t)
-	 (remote-config
-	  (expand-file-name
-	   tramp-crypt-encfs-config (tramp-crypt-get-remote-dir vec)))
-	 (local-config (tramp-crypt-config-file-name vec)))
-    ;; There is no local encfs6 config file.
-    (unless (file-exists-p local-config)
-      (if (and tramp-crypt-save-encfs-config-remote
-	       (file-exists-p remote-config))
-	  ;; Copy remote encfs6 config file if possible.
-	  (copy-file remote-config local-config 'ok 'keep)
+    ;; The following operations must be performed without
+    ;; `tramp-crypt-file-name-handler'.
+    (let* (tramp-crypt-enabled
+	   ;; Don't check for a proper method.
+	   (non-essential t)
+	   (remote-config
+	    (expand-file-name
+	     tramp-crypt-encfs-config (tramp-crypt-get-remote-dir vec)))
+	   (local-config (tramp-crypt-config-file-name vec)))
+      ;; There is no local encfs6 config file.
+      (unless (file-exists-p local-config)
+	(if (and tramp-crypt-save-encfs-config-remote
+		 (file-exists-p remote-config))
+	    ;; Copy remote encfs6 config file if possible.
+	    (copy-file remote-config local-config 'ok 'keep)
 
-	;; Create local encfs6 config file otherwise.
-	(let* ((default-directory tramp-compat-temporary-file-directory)
-	       (tmpdir1 (file-name-as-directory
-			 (tramp-compat-make-temp-file " .crypt" 'dir-flag)))
-	       (tmpdir2 (file-name-as-directory
-			 (tramp-compat-make-temp-file " .nocrypt" 'dir-flag))))
-	  ;; Enable `auth-source', unless "emacs -Q" has been called.
-	  (tramp-set-connection-property
-	   vec "first-password-request" tramp-cache-read-persistent-data)
-	  (with-temp-buffer
-	    (insert
-	     (tramp-read-passwd
-	      (tramp-get-connection-process vec)
-	      (format
-	       "New EncFS Password for %s " (tramp-crypt-get-remote-dir vec))))
-	    (when
-		(zerop
-		 (tramp-call-process-region
-		  vec (point-min) (point-max)
-		  tramp-crypt-encfs-program nil (tramp-get-connection-buffer vec)
-		  nil tramp-crypt-encfs-option "--extpass=cat" tmpdir1 tmpdir2))
-	      ;; Save the password.
-	      (ignore-errors
-		(and (functionp tramp-password-save-function)
-		     (funcall tramp-password-save-function)))))
+	  ;; Create local encfs6 config file otherwise.
+	  (let* ((default-directory tramp-compat-temporary-file-directory)
+		 (tmpdir1 (file-name-as-directory
+			   (tramp-compat-make-temp-file " .crypt" 'dir-flag)))
+		 (tmpdir2 (file-name-as-directory
+			   (tramp-compat-make-temp-file " .nocrypt" 'dir-flag))))
+	    ;; Enable `auth-source', unless "emacs -Q" has been called.
+	    (tramp-set-connection-property
+	     vec "first-password-request" tramp-cache-read-persistent-data)
+	    (with-temp-buffer
+	      (insert
+	       (tramp-read-passwd
+		(tramp-get-connection-process vec)
+		(format
+		 "New EncFS Password for %s " (tramp-crypt-get-remote-dir vec))))
+	      (when
+		  (zerop
+		   (tramp-call-process-region
+		    vec (point-min) (point-max)
+		    tramp-crypt-encfs-program nil
+		    (tramp-get-connection-buffer vec) nil
+		    tramp-crypt-encfs-option "--extpass=cat" tmpdir1 tmpdir2))
+		;; Save the password.
+		(ignore-errors
+		  (and (functionp tramp-password-save-function)
+		       (funcall tramp-password-save-function)))))
 
-	  ;; Write local config file.  Suppress file name IV chaining mode.
-	  (with-temp-file local-config
-	    (insert-file-contents
-	     (expand-file-name tramp-crypt-encfs-config tmpdir1))
-	    (when (search-forward
-		   "<chainedNameIV>1</chainedNameIV>" nil 'noerror)
-	      (replace-match "<chainedNameIV>0</chainedNameIV>")))
+	    ;; Write local config file.  Suppress file name IV chaining mode.
+	    (with-temp-file local-config
+	      (insert-file-contents
+	       (expand-file-name tramp-crypt-encfs-config tmpdir1))
+	      (when (search-forward
+		     "<chainedNameIV>1</chainedNameIV>" nil 'noerror)
+		(replace-match "<chainedNameIV>0</chainedNameIV>")))
 
-	  ;; Unmount encfs.  Delete temporary directories.
-	  (tramp-call-process
-	   vec tramp-crypt-encfs-program nil nil nil
-	   "--unmount" tmpdir1 tmpdir2)
-	  (delete-directory tmpdir1 'recursive)
-	  (delete-directory tmpdir2)
+	    ;; Unmount encfs.  Delete temporary directories.
+	    (tramp-call-process
+	     vec tramp-crypt-encfs-program nil nil nil
+	     "--unmount" tmpdir1 tmpdir2)
+	    (delete-directory tmpdir1 'recursive)
+	    (delete-directory tmpdir2)
 
-	  ;; Copy local encfs6 config file to remote.
-	  (when tramp-crypt-save-encfs-config-remote
-	    (copy-file local-config remote-config 'ok 'keep)))))))
+	    ;; Copy local encfs6 config file to remote.
+	    (when tramp-crypt-save-encfs-config-remote
+	      (copy-file local-config remote-config 'ok 'keep))))))))
 
 (defun tramp-crypt-send-command (vec &rest args)
   "Send encfsctl command to connection VEC.
@@ -499,7 +504,7 @@ directory.  File names will be also encrypted."
     (tramp-user-error nil "Feature is not enabled"))
   (unless (and (tramp-tramp-file-p name) (file-directory-p name))
     (tramp-user-error nil "%s must be an existing remote directory" name))
-  (when (tramp-compat-file-name-quoted-p name)
+  (when (file-name-quoted-p name)
     (tramp-user-error nil "%s must not be quoted" name))
   (setq name (file-name-as-directory (expand-file-name name)))
   (unless (member name tramp-crypt-directories)
@@ -558,7 +563,7 @@ localname."
 (defun tramp-crypt-handle-access-file (filename string)
   "Like `access-file' for Tramp files."
   (let* ((encrypt-filename (tramp-crypt-encrypt-file-name filename))
-	 (encrypt-regexp (tramp-compat-rx (literal encrypt-filename) eos))
+	 (encrypt-regexp (rx (literal encrypt-filename) eos))
 	 tramp-crypt-enabled)
     (condition-case err
 	(access-file encrypt-filename string)
@@ -691,17 +696,17 @@ absolute file names."
     (directory &optional recursive _trash)
   "Like `delete-directory' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name directory) nil
-    (tramp-flush-directory-properties v localname)
     (let (tramp-crypt-enabled)
-      (delete-directory (tramp-crypt-encrypt-file-name directory) recursive))))
+      (delete-directory (tramp-crypt-encrypt-file-name directory) recursive))
+    (tramp-flush-directory-properties v localname)))
 
 ;; Encrypted files won't be trashed.
 (defun tramp-crypt-handle-delete-file (filename &optional _trash)
   "Like `delete-file' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
-    (tramp-flush-file-properties v localname)
     (let (tramp-crypt-enabled)
-      (delete-file (tramp-crypt-encrypt-file-name filename)))))
+      (delete-file (tramp-crypt-encrypt-file-name filename)))
+    (tramp-flush-file-properties v localname)))
 
 (defun tramp-crypt-handle-directory-files
     (directory &optional full match nosort count)
@@ -711,8 +716,7 @@ absolute file names."
       (mapcar
        (lambda (x)
 	 (replace-regexp-in-string
-	  (tramp-compat-rx bos (literal directory)) ""
-	  (tramp-crypt-decrypt-file-name x)))
+	  (rx bos (literal directory)) "" (tramp-crypt-decrypt-file-name x)))
        (directory-files (tramp-crypt-encrypt-file-name directory) 'full)))))
 
 (defun tramp-crypt-handle-file-attributes (filename &optional id-format)
@@ -764,9 +768,7 @@ absolute file names."
 (defun tramp-crypt-handle-file-system-info (filename)
   "Like `file-system-info' for Tramp files."
   (let (tramp-crypt-enabled)
-    ;; `file-system-info' exists since Emacs 27.1.
-    (tramp-compat-funcall
-     'file-system-info (tramp-crypt-encrypt-file-name filename))))
+    (file-system-info (tramp-crypt-encrypt-file-name filename))))
 
 (defun tramp-crypt-handle-file-writable-p (filename)
   "Like `file-writable-p' for Tramp files."
@@ -777,27 +779,26 @@ absolute file names."
   (filename switches &optional wildcard full-directory-p)
   "Like `insert-directory' for Tramp files.
 WILDCARD is not supported."
-  ;; This package has been added to Emacs 27.1.
-  (when (load "text-property-search" 'noerror 'nomessage)
-    (let (tramp-crypt-enabled)
-      (tramp-handle-insert-directory
-       (tramp-crypt-encrypt-file-name filename)
-       switches wildcard full-directory-p)
-      (let* ((filename (file-name-as-directory filename))
-	     (enc (tramp-crypt-encrypt-file-name filename))
-	     match string)
-	(goto-char (point-min))
-	(while (setq match (text-property-search-forward 'dired-filename t t))
-	  (setq string
-		(buffer-substring
-		 (prop-match-beginning match) (prop-match-end match))
-		string (if (file-name-absolute-p string)
-			   (tramp-crypt-decrypt-file-name string)
-			 (substring
-			  (tramp-crypt-decrypt-file-name (concat enc string))
-			  (length filename))))
-	  (delete-region (prop-match-beginning match) (prop-match-end match))
-	  (insert (propertize string 'dired-filename t)))))))
+  (require 'text-property-search)
+  (let (tramp-crypt-enabled)
+    (tramp-handle-insert-directory
+     (tramp-crypt-encrypt-file-name filename)
+     switches wildcard full-directory-p)
+    (let* ((filename (file-name-as-directory filename))
+	   (enc (tramp-crypt-encrypt-file-name filename))
+	   match string)
+      (goto-char (point-min))
+      (while (setq match (text-property-search-forward 'dired-filename t t))
+	(setq string
+	      (buffer-substring
+	       (prop-match-beginning match) (prop-match-end match))
+	      string (if (file-name-absolute-p string)
+			 (tramp-crypt-decrypt-file-name string)
+		       (substring
+			(tramp-crypt-decrypt-file-name (concat enc string))
+			(length filename))))
+	(delete-region (prop-match-beginning match) (prop-match-end match))
+	(insert (propertize string 'dired-filename t))))))
 
 (defun tramp-crypt-handle-lock-file (filename)
   "Like `lock-file' for Tramp files."
@@ -808,16 +809,9 @@ WILDCARD is not supported."
 
 (defun tramp-crypt-handle-make-directory (dir &optional parents)
   "Like `make-directory' for Tramp files."
-  (with-parsed-tramp-file-name (expand-file-name dir) nil
-    (when (and (null parents) (file-exists-p dir))
-      (tramp-error v 'file-already-exists dir))
+  (tramp-skeleton-make-directory dir parents
     (let (tramp-crypt-enabled)
-      (make-directory (tramp-crypt-encrypt-file-name dir) parents))
-    ;; When PARENTS is non-nil, DIR could be a chain of non-existent
-    ;; directories a/b/c/...  Instead of checking, we simply flush the
-    ;; whole cache.
-    (tramp-flush-directory-properties
-     v (if parents "/" (file-name-directory localname)))))
+      (make-directory (tramp-crypt-encrypt-file-name dir) parents))))
 
 (defun tramp-crypt-handle-rename-file
   (filename newname &optional ok-if-already-exists)
@@ -862,10 +856,11 @@ WILDCARD is not supported."
      'unlock-file (tramp-crypt-encrypt-file-name filename))))
 
 (defun tramp-crypt-cleanup-connection (vec)
-  "Cleanup crypt ressources determined by VEC."
+  "Cleanup crypt resources determined by VEC."
   (let ((tramp-cleanup-connection-hook
 	 (remove
-	  #'tramp-crypt-cleanup-connection tramp-cleanup-connection-hook)))
+	  #'tramp-crypt-cleanup-connection tramp-cleanup-connection-hook))
+	(tramp-crypt-enabled t))
     (dolist (dir tramp-crypt-directories)
       (when (tramp-file-name-equal-p vec (tramp-dissect-file-name dir))
 	(tramp-cleanup-connection (tramp-crypt-dissect-file-name dir))))))
