@@ -101,6 +101,7 @@
     (file-equal-p . tramp-handle-file-equal-p)
     (file-executable-p . tramp-fuse-handle-file-executable-p)
     (file-exists-p . tramp-fuse-handle-file-exists-p)
+    (file-group-gid . tramp-handle-file-group-gid)
     (file-in-directory-p . tramp-handle-file-in-directory-p)
     (file-local-copy . tramp-handle-file-local-copy)
     (file-locked-p . tramp-handle-file-locked-p)
@@ -124,6 +125,7 @@
     (file-symlink-p . tramp-handle-file-symlink-p)
     (file-system-info . tramp-sshfs-handle-file-system-info)
     (file-truename . tramp-handle-file-truename)
+    (file-user-uid . tramp-handle-file-user-uid)
     (file-writable-p . tramp-sshfs-handle-file-writable-p)
     (find-backup-file-name . tramp-handle-find-backup-file-name)
     ;; `get-file-buffer' performed by default handler.
@@ -179,8 +181,10 @@ Operations not mentioned here will be handled by the default Emacs primitives.")
 First arg specifies the OPERATION, second arg is a list of
 arguments to pass to the OPERATION."
   (if-let ((fn (assoc operation tramp-sshfs-file-name-handler-alist)))
-      (save-match-data (apply (cdr fn) args))
-    (tramp-run-real-handler operation args)))
+      (prog1 (save-match-data (apply (cdr fn) args))
+	(setq tramp-debug-message-fnh-function (cdr fn)))
+    (prog1 (tramp-run-real-handler operation args)
+      (setq tramp-debug-message-fnh-function operation))))
 
 ;;;###tramp-autoload
 (tramp--with-startup
@@ -228,8 +232,7 @@ arguments to pass to the OPERATION."
 
 (defun tramp-sshfs-handle-file-system-info (filename)
   "Like `file-system-info' for Tramp files."
-  ;;`file-system-info' exists since Emacs 27.1.
-  (tramp-compat-funcall 'file-system-info (tramp-fuse-local-file-name filename)))
+  (file-system-info (tramp-fuse-local-file-name filename)))
 
 (defun tramp-sshfs-handle-file-writable-p (filename)
   "Like `file-writable-p' for Tramp files."
@@ -266,7 +269,7 @@ arguments to pass to the OPERATION."
       ;; Determine input.
       (if (null infile)
 	  (setq input (tramp-get-remote-null-device v))
-	(setq infile (tramp-compat-file-name-unquote (expand-file-name infile)))
+	(setq infile (file-name-unquote (expand-file-name infile)))
 	(if (tramp-equal-remote default-directory infile)
 	    ;; INFILE is on the same remote host.
 	    (setq input (tramp-unquote-file-local-name infile))
@@ -394,53 +397,53 @@ connection if a previous connection has died for some reason."
   (unless (tramp-connectable-p vec)
     (throw 'non-essential 'non-essential))
 
-  ;; We need a process bound to the connection buffer.  Therefore, we
-  ;; create a dummy process.  Maybe there is a better solution?
-  (unless (get-buffer-process (tramp-get-connection-buffer vec))
-    (let ((p (make-network-process
-	      :name (tramp-get-connection-name vec)
-	      :buffer (tramp-get-connection-buffer vec)
-	      :server t :host 'local :service t :noquery t)))
-      (process-put p 'tramp-vector vec)
-      (set-process-query-on-exit-flag p nil)
+  (with-tramp-debug-message vec "Opening connection"
+    ;; We need a process bound to the connection buffer.  Therefore,
+    ;; we create a dummy process.  Maybe there is a better solution?
+    (unless (get-buffer-process (tramp-get-connection-buffer vec))
+      (let ((p (make-network-process
+		:name (tramp-get-connection-name vec)
+		:buffer (tramp-get-connection-buffer vec)
+		:server t :host 'local :service t :noquery t)))
+	(tramp-post-process-creation p vec)
 
-      ;; Set connection-local variables.
-      (tramp-set-connection-local-variables vec)))
+	;; Set connection-local variables.
+	(tramp-set-connection-local-variables vec)))
 
-  ;; Create directory.
-  (unless (file-directory-p (tramp-fuse-mount-point vec))
-    (make-directory (tramp-fuse-mount-point vec) 'parents))
+    ;; Create directory.
+    (unless (file-directory-p (tramp-fuse-mount-point vec))
+      (make-directory (tramp-fuse-mount-point vec) 'parents))
 
-  (unless
-      (or (tramp-fuse-mounted-p vec)
-	  (with-temp-buffer
-	    (zerop
-	     (apply
-	      #'tramp-call-process
-	      vec tramp-sshfs-program nil t nil
-	      (tramp-fuse-mount-spec vec)
-	      (tramp-fuse-mount-point vec)
-	      (tramp-expand-args
-	       vec 'tramp-mount-args
-	       ?p (or (tramp-file-name-port vec) ""))))))
-    (tramp-error
-     vec 'file-error "Error mounting %s" (tramp-fuse-mount-spec vec)))
+    (unless
+	(or (tramp-fuse-mounted-p vec)
+	    (with-temp-buffer
+	      (zerop
+	       (apply
+		#'tramp-call-process
+		vec tramp-sshfs-program nil t nil
+		(tramp-fuse-mount-spec vec)
+		(tramp-fuse-mount-point vec)
+		(tramp-expand-args
+		 vec 'tramp-mount-args
+		 ?p (or (tramp-file-name-port vec) ""))))))
+      (tramp-error
+       vec 'file-error "Error mounting %s" (tramp-fuse-mount-spec vec)))
 
-  ;; Mark it as connected.
-  (add-to-list 'tramp-fuse-mount-points (tramp-file-name-unify vec))
-  (tramp-set-connection-property
-   (tramp-get-connection-process vec) "connected" t)
+    ;; Mark it as connected.
+    (add-to-list 'tramp-fuse-mount-points (tramp-file-name-unify vec))
+    (tramp-set-connection-property
+     (tramp-get-connection-process vec) "connected" t)
 
-  ;; In `tramp-check-cached-permissions', the connection properties
-  ;; "{uid,gid}-{integer,string}" are used.  We set them to proper values.
-  (with-tramp-connection-property
-      vec "uid-integer" (tramp-get-local-uid 'integer))
-  (with-tramp-connection-property
-      vec "gid-integer" (tramp-get-local-gid 'integer))
-  (with-tramp-connection-property
-      vec "uid-string" (tramp-get-local-uid 'string))
-  (with-tramp-connection-property
-      vec "gid-string" (tramp-get-local-gid 'string)))
+    ;; In `tramp-check-cached-permissions', the connection properties
+    ;; "{uid,gid}-{integer,string}" are used.  We set them to proper values.
+    (with-tramp-connection-property
+	vec "uid-integer" (tramp-get-local-uid 'integer))
+    (with-tramp-connection-property
+	vec "gid-integer" (tramp-get-local-gid 'integer))
+    (with-tramp-connection-property
+	vec "uid-string" (tramp-get-local-uid 'string))
+    (with-tramp-connection-property
+	vec "gid-string" (tramp-get-local-gid 'string))))
 
 ;; `shell-mode' tries to open remote files like "/sshfs:user@host:~/.history".
 ;; This fails, because the tilde cannot be expanded.  Tell
